@@ -4,6 +4,7 @@ import (
 	"errors"
 	"flag"
 	"fmt"
+	"io/fs"
 	"os"
 	"os/user"
 	"path"
@@ -34,10 +35,6 @@ var (
 	versionFlag bool // True if -version is used
 )
 
-const (
-	SystemUserDirLevel = 5
-)
-
 var (
 	// data saved between logToKmsg calls
 	noKmsg   = false
@@ -106,103 +103,207 @@ func Debugf(format string, a ...interface{}) {
 	}
 }
 
+type searchPaths struct {
+	sorted []string
+	// map to store paths so we can quickly check if we saw them already and not loop in case of symlinks
+	visitedDirs map[string]struct{}
+}
+
+func newSearchPaths() *searchPaths {
+	return &searchPaths{
+		sorted:      make([]string, 0),
+		visitedDirs: make(map[string]struct{}, 0),
+	}
+}
+
+func (s *searchPaths) Add(path string) {
+	s.sorted = append(s.sorted, path)
+	s.visitedDirs[path] = struct{}{}
+}
+
+func (s *searchPaths) Visited(path string) bool {
+	_, visited := s.visitedDirs[path]
+	return visited
+}
+
 // This returns the directories where we read quadlet .container and .volumes from
 // For system generators these are in /usr/share/containers/systemd (for distro files)
 // and /etc/containers/systemd (for sysadmin files).
-// For user generators these can live in /etc/containers/systemd/users, /etc/containers/systemd/users/$UID, and $XDG_CONFIG_HOME/containers/systemd
+// For user generators these can live in $XDG_RUNTIME_DIR/containers/systemd, /etc/containers/systemd/users, /etc/containers/systemd/users/$UID, and $XDG_CONFIG_HOME/containers/systemd
 func getUnitDirs(rootless bool) []string {
-	// Allow overriding source dir, this is mainly for the CI tests
-	unitDirsEnv := os.Getenv("QUADLET_UNIT_DIRS")
-	dirs := make([]string, 0)
+	paths := newSearchPaths()
 
-	if len(unitDirsEnv) > 0 {
-		for _, eachUnitDir := range strings.Split(unitDirsEnv, ":") {
-			if !filepath.IsAbs(eachUnitDir) {
-				Logf("%s not a valid file path", eachUnitDir)
-				return nil
-			}
-			dirs = appendSubPaths(dirs, eachUnitDir, false, nil)
-		}
-		return dirs
+	// Allow overriding source dir, this is mainly for the CI tests
+	if getDirsFromEnv(paths) {
+		return paths.sorted
 	}
 
+	resolvedUnitDirAdminUser := resolveUnitDirAdminUser()
+	userLevelFilter := getUserLevelFilter(resolvedUnitDirAdminUser)
+
 	if rootless {
-		configDir, err := os.UserConfigDir()
-		if err != nil {
-			fmt.Fprintf(os.Stderr, "Warning: %v", err)
-			return nil
-		}
-		dirs = appendSubPaths(dirs, path.Join(configDir, "containers/systemd"), false, nil)
-		u, err := user.Current()
-		if err == nil {
-			dirs = appendSubPaths(dirs, filepath.Join(quadlet.UnitDirAdmin, "users"), true, nonNumericFilter)
-			dirs = appendSubPaths(dirs, filepath.Join(quadlet.UnitDirAdmin, "users", u.Uid), true, userLevelFilter)
-		} else {
-			fmt.Fprintf(os.Stderr, "Warning: %v", err)
-		}
-		return append(dirs, filepath.Join(quadlet.UnitDirAdmin, "users"))
+		systemUserDirLevel := len(strings.Split(resolvedUnitDirAdminUser, string(os.PathSeparator)))
+		nonNumericFilter := getNonNumericFilter(resolvedUnitDirAdminUser, systemUserDirLevel)
+		getRootlessDirs(paths, nonNumericFilter, userLevelFilter)
+	} else {
+		getRootDirs(paths, userLevelFilter)
 	}
-
-	dirs = appendSubPaths(dirs, quadlet.UnitDirAdmin, false, userLevelFilter)
-	return appendSubPaths(dirs, quadlet.UnitDirDistro, false, nil)
+	return paths.sorted
 }
 
-func appendSubPaths(dirs []string, path string, isUserFlag bool, filterPtr func(string, bool) bool) []string {
-	resolvedPath, err := filepath.EvalSymlinks(path)
-	if err != nil {
-		Debugf("Error occurred resolving path %q: %s", path, err)
-		// Despite the failure add the path to the list for logging purposes
-		// This is the equivalent of adding the path when info==nil below
-		dirs = append(dirs, path)
-		return dirs
+func getDirsFromEnv(paths *searchPaths) bool {
+	unitDirsEnv := os.Getenv("QUADLET_UNIT_DIRS")
+	if len(unitDirsEnv) == 0 {
+		return false
 	}
 
-	err = filepath.WalkDir(resolvedPath, func(_path string, info os.DirEntry, err error) error {
-		// Ignore drop-in directory subpaths
-		if !strings.HasSuffix(_path, ".d") {
-			if info == nil || info.IsDir() {
-				if filterPtr == nil || filterPtr(_path, isUserFlag) {
-					dirs = append(dirs, _path)
-				}
-			}
+	for _, eachUnitDir := range strings.Split(unitDirsEnv, ":") {
+		if !filepath.IsAbs(eachUnitDir) {
+			Logf("%s not a valid file path", eachUnitDir)
+			break
 		}
-		return err
-	})
+		appendSubPaths(paths, eachUnitDir, false, nil)
+	}
+	return true
+}
+
+func getRootlessDirs(paths *searchPaths, nonNumericFilter, userLevelFilter func(string, bool) bool) {
+	runtimeDir, found := os.LookupEnv("XDG_RUNTIME_DIR")
+	if found {
+		appendSubPaths(paths, path.Join(runtimeDir, "containers/systemd"), false, nil)
+	}
+
+	configDir, err := os.UserConfigDir()
+	if err != nil {
+		fmt.Fprintf(os.Stderr, "Warning: %v", err)
+		return
+	}
+	appendSubPaths(paths, path.Join(configDir, "containers/systemd"), false, nil)
+
+	u, err := user.Current()
+	if err == nil {
+		appendSubPaths(paths, filepath.Join(quadlet.UnitDirAdmin, "users"), true, nonNumericFilter)
+		appendSubPaths(paths, filepath.Join(quadlet.UnitDirAdmin, "users", u.Uid), true, userLevelFilter)
+	} else {
+		fmt.Fprintf(os.Stderr, "Warning: %v", err)
+	}
+
+	paths.Add(filepath.Join(quadlet.UnitDirAdmin, "users"))
+}
+
+func getRootDirs(paths *searchPaths, userLevelFilter func(string, bool) bool) {
+	appendSubPaths(paths, quadlet.UnitDirTemp, false, userLevelFilter)
+	appendSubPaths(paths, quadlet.UnitDirAdmin, false, userLevelFilter)
+	appendSubPaths(paths, quadlet.UnitDirDistro, false, nil)
+}
+
+func resolveUnitDirAdminUser() string {
+	unitDirAdminUser := filepath.Join(quadlet.UnitDirAdmin, "users")
+	var err error
+	var resolvedUnitDirAdminUser string
+	if resolvedUnitDirAdminUser, err = filepath.EvalSymlinks(unitDirAdminUser); err != nil {
+		if !errors.Is(err, fs.ErrNotExist) {
+			Debugf("Error occurred resolving path %q: %s", unitDirAdminUser, err)
+		}
+		resolvedUnitDirAdminUser = unitDirAdminUser
+	}
+	return resolvedUnitDirAdminUser
+}
+
+func appendSubPaths(paths *searchPaths, path string, isUserFlag bool, filterPtr func(string, bool) bool) {
+	resolvedPath, err := filepath.EvalSymlinks(path)
+	if err != nil {
+		if !errors.Is(err, fs.ErrNotExist) {
+			Debugf("Error occurred resolving path %q: %s", path, err)
+		}
+		// Despite the failure add the path to the list for logging purposes
+		// This is the equivalent of adding the path when info==nil below
+		paths.Add(path)
+		return
+	}
+
+	if skipPath(paths, resolvedPath, isUserFlag, filterPtr) {
+		return
+	}
+
+	// Add the current directory
+	paths.Add(resolvedPath)
+
+	// Read the contents of the directory
+	entries, err := os.ReadDir(resolvedPath)
 	if err != nil {
 		if !errors.Is(err, os.ErrNotExist) {
 			Debugf("Error occurred walking sub directories %q: %s", path, err)
 		}
+		return
+	}
+
+	// Recursively run through the contents of the directory
+	for _, entry := range entries {
+		fullPath := filepath.Join(resolvedPath, entry.Name())
+		appendSubPaths(paths, fullPath, isUserFlag, filterPtr)
 	}
-	return dirs
 }
 
-func nonNumericFilter(_path string, isUserFlag bool) bool {
-	// when running in rootless, recursive walk directories that are non numeric
-	// ignore sub dirs under the `users` directory which correspond to a user id
-	if strings.Contains(_path, filepath.Join(quadlet.UnitDirAdmin, "users")) {
-		listDirUserPathLevels := strings.Split(_path, string(os.PathSeparator))
-		if len(listDirUserPathLevels) > SystemUserDirLevel {
-			if !(regexp.MustCompile(`^[0-9]*$`).MatchString(listDirUserPathLevels[SystemUserDirLevel])) {
-				return true
-			}
-		}
-	} else {
+func skipPath(paths *searchPaths, path string, isUserFlag bool, filterPtr func(string, bool) bool) bool {
+	// If the path is already in the map no need to read it again
+	if paths.Visited(path) {
 		return true
 	}
-	return false
+
+	// Don't traverse drop-in directories
+	if strings.HasSuffix(path, ".d") {
+		return true
+	}
+
+	// Check if the directory should be filtered out
+	if filterPtr != nil && !filterPtr(path, isUserFlag) {
+		return true
+	}
+
+	stat, err := os.Stat(path)
+	if err != nil {
+		if !errors.Is(err, fs.ErrNotExist) {
+			Debugf("Error occurred resolving path %q: %s", path, err)
+		}
+		return true
+	}
+
+	// Not a directory nothing to add
+	return !stat.IsDir()
 }
 
-func userLevelFilter(_path string, isUserFlag bool) bool {
-	// if quadlet generator is run rootless, do not recurse other user sub dirs
-	// if quadlet generator is run as root, ignore users sub dirs
-	if strings.Contains(_path, filepath.Join(quadlet.UnitDirAdmin, "users")) {
-		if isUserFlag {
+func getNonNumericFilter(resolvedUnitDirAdminUser string, systemUserDirLevel int) func(string, bool) bool {
+	return func(path string, isUserFlag bool) bool {
+		// when running in rootless, recursive walk directories that are non numeric
+		// ignore sub dirs under the `users` directory which correspond to a user id
+		if strings.HasPrefix(path, resolvedUnitDirAdminUser) {
+			listDirUserPathLevels := strings.Split(path, string(os.PathSeparator))
+			if len(listDirUserPathLevels) > systemUserDirLevel {
+				if !(regexp.MustCompile(`^[0-9]*$`).MatchString(listDirUserPathLevels[systemUserDirLevel])) {
+					return true
+				}
+			}
+		} else {
 			return true
 		}
-	} else {
-		return true
+		return false
+	}
+}
+
+func getUserLevelFilter(resolvedUnitDirAdminUser string) func(string, bool) bool {
+	return func(_path string, isUserFlag bool) bool {
+		// if quadlet generator is run rootless, do not recurse other user sub dirs
+		// if quadlet generator is run as root, ignore users sub dirs
+		if strings.HasPrefix(_path, resolvedUnitDirAdminUser) {
+			if isUserFlag {
+				return true
+			}
+		} else {
+			return true
+		}
+		return false
 	}
-	return false
 }
 
 func isExtSupported(filename string) bool {
@@ -480,34 +581,47 @@ func warnIfAmbiguousName(unit *parser.UnitFile, group string) {
 	}
 }
 
-func generatePodsInfoMap(units []*parser.UnitFile) map[string]*quadlet.PodInfo {
-	podsInfoMap := make(map[string]*quadlet.PodInfo)
+func generateUnitsInfoMap(units []*parser.UnitFile) map[string]*quadlet.UnitInfo {
+	unitsInfoMap := make(map[string]*quadlet.UnitInfo)
 	for _, unit := range units {
-		if !strings.HasSuffix(unit.Filename, ".pod") {
+		var serviceName string
+		var containers []string
+		var resourceName string
+
+		switch {
+		case strings.HasSuffix(unit.Filename, ".container"):
+			serviceName = quadlet.GetContainerServiceName(unit)
+		case strings.HasSuffix(unit.Filename, ".volume"):
+			serviceName = quadlet.GetVolumeServiceName(unit)
+		case strings.HasSuffix(unit.Filename, ".kube"):
+			serviceName = quadlet.GetKubeServiceName(unit)
+		case strings.HasSuffix(unit.Filename, ".network"):
+			serviceName = quadlet.GetNetworkServiceName(unit)
+		case strings.HasSuffix(unit.Filename, ".image"):
+			serviceName = quadlet.GetImageServiceName(unit)
+		case strings.HasSuffix(unit.Filename, ".build"):
+			serviceName = quadlet.GetBuildServiceName(unit)
+			// Prefill resouceNames for .build files. This is significantly less complex than
+			// pre-computing all resourceNames for all Quadlet types (which is rather complex for a few
+			// types), but still breaks the dependency cycle between .volume and .build ([Volume] can
+			// have Image=some.build, and [Build] can have Volume=some.volume:/some-volume)
+			resourceName = quadlet.GetBuiltImageName(unit)
+		case strings.HasSuffix(unit.Filename, ".pod"):
+			serviceName = quadlet.GetPodServiceName(unit)
+			containers = make([]string, 0)
+		default:
+			Logf("Unsupported file type %q", unit.Filename)
 			continue
 		}
 
-		serviceName := quadlet.GetPodServiceName(unit)
-		podsInfoMap[unit.Filename] = &quadlet.PodInfo{
-			ServiceName: serviceName,
-			Containers:  make([]string, 0),
+		unitsInfoMap[unit.Filename] = &quadlet.UnitInfo{
+			ServiceName:       serviceName,
+			ContainersToStart: containers,
+			ResourceName:      resourceName,
 		}
 	}
 
-	return podsInfoMap
-}
-
-func prefillBuiltImageNames(units []*parser.UnitFile, resourceNames map[string]string) {
-	for _, unit := range units {
-		if !strings.HasSuffix(unit.Filename, ".build") {
-			continue
-		}
-
-		imageName := quadlet.GetBuiltImageName(unit)
-		if len(imageName) > 0 {
-			resourceNames[unit.Filename] = imageName
-		}
-	}
+	return unitsInfoMap
 }
 
 func main() {
@@ -559,10 +673,10 @@ func process() error {
 		Debugf("Starting quadlet-generator, output to: %s", outputPath)
 	}
 
-	sourcePaths := getUnitDirs(isUserFlag)
+	sourcePathsMap := getUnitDirs(isUserFlag)
 
 	var units []*parser.UnitFile
-	for _, d := range sourcePaths {
+	for _, d := range sourcePathsMap {
 		if result, err := loadUnitsFromDir(d); err != nil {
 			reportError(err)
 		} else {
@@ -573,12 +687,12 @@ func process() error {
 	if len(units) == 0 {
 		// containers/podman/issues/17374: exit cleanly but log that we
 		// had nothing to do
-		Debugf("No files parsed from %s", sourcePaths)
+		Debugf("No files parsed from %s", sourcePathsMap)
 		return prevError
 	}
 
 	for _, unit := range units {
-		if err := loadUnitDropins(unit, sourcePaths); err != nil {
+		if err := loadUnitDropins(unit, sourcePathsMap); err != nil {
 			reportError(err)
 		}
 	}
@@ -606,40 +720,30 @@ func process() error {
 	})
 
 	// Generate the PodsInfoMap to allow containers to link to their pods and add themselves to the pod's containers list
-	podsInfoMap := generatePodsInfoMap(units)
-
-	// A map of network/volume unit file-names, against their calculated names, as needed by Podman.
-	var resourceNames = make(map[string]string)
-
-	// Prefill resouceNames for .build files. This is significantly less complex than
-	// pre-computing all resourceNames for all Quadlet types (which is rather complex for a few
-	// types), but still breaks the dependency cycle between .volume and .build ([Volume] can
-	// have Image=some.build, and [Build] can have Volume=some.volume:/some-volume)
-	prefillBuiltImageNames(units, resourceNames)
+	unitsInfoMap := generateUnitsInfoMap(units)
 
 	for _, unit := range units {
 		var service *parser.UnitFile
-		var name string
 		var err error
 
 		switch {
 		case strings.HasSuffix(unit.Filename, ".container"):
 			warnIfAmbiguousName(unit, quadlet.ContainerGroup)
-			service, err = quadlet.ConvertContainer(unit, resourceNames, isUserFlag, podsInfoMap)
+			service, err = quadlet.ConvertContainer(unit, isUserFlag, unitsInfoMap)
 		case strings.HasSuffix(unit.Filename, ".volume"):
 			warnIfAmbiguousName(unit, quadlet.VolumeGroup)
-			service, name, err = quadlet.ConvertVolume(unit, unit.Filename, resourceNames)
+			service, err = quadlet.ConvertVolume(unit, unit.Filename, unitsInfoMap, isUserFlag)
 		case strings.HasSuffix(unit.Filename, ".kube"):
-			service, err = quadlet.ConvertKube(unit, resourceNames, isUserFlag)
+			service, err = quadlet.ConvertKube(unit, unitsInfoMap, isUserFlag)
 		case strings.HasSuffix(unit.Filename, ".network"):
-			service, name, err = quadlet.ConvertNetwork(unit, unit.Filename)
+			service, err = quadlet.ConvertNetwork(unit, unit.Filename, unitsInfoMap, isUserFlag)
 		case strings.HasSuffix(unit.Filename, ".image"):
 			warnIfAmbiguousName(unit, quadlet.ImageGroup)
-			service, name, err = quadlet.ConvertImage(unit)
+			service, err = quadlet.ConvertImage(unit, unitsInfoMap, isUserFlag)
 		case strings.HasSuffix(unit.Filename, ".build"):
-			service, name, err = quadlet.ConvertBuild(unit, resourceNames)
+			service, err = quadlet.ConvertBuild(unit, unitsInfoMap, isUserFlag)
 		case strings.HasSuffix(unit.Filename, ".pod"):
-			service, err = quadlet.ConvertPod(unit, unit.Filename, podsInfoMap, resourceNames)
+			service, err = quadlet.ConvertPod(unit, unit.Filename, unitsInfoMap, isUserFlag)
 		default:
 			Logf("Unsupported file type %q", unit.Filename)
 			continue
@@ -650,9 +754,6 @@ func process() error {
 			continue
 		}
 
-		if name != "" {
-			resourceNames[unit.Filename] = name
-		}
 		service.Path = path.Join(outputPath, service.Filename)
 
 		if dryRunFlag {
