@@ -1,29 +1,25 @@
+//go:build !remote
+
 package libpod
 
 import (
 	"bufio"
+	"bytes"
 	"context"
 	"errors"
 	"fmt"
+	"io/fs"
 	"os"
 	"path/filepath"
 	"strings"
 	"time"
 
-	"github.com/containers/podman/v4/libpod/define"
-	"github.com/containers/podman/v4/libpod/events"
+	"github.com/containers/podman/v5/libpod/define"
+	"github.com/containers/podman/v5/libpod/shutdown"
 	"github.com/sirupsen/logrus"
 	"golang.org/x/sys/unix"
 )
 
-const (
-	// MaxHealthCheckNumberLogs is the maximum number of attempts we keep
-	// in the healthcheck history file
-	MaxHealthCheckNumberLogs int = 5
-	// MaxHealthCheckLogLength in characters
-	MaxHealthCheckLogLength = 500
-)
-
 // HealthCheck verifies the state and validity of the healthcheck configuration
 // on the container and then executes the healthcheck
 func (r *Runtime) HealthCheck(ctx context.Context, name string) (define.HealthCheckStatus, error) {
@@ -85,30 +81,17 @@ func (c *Container) runHealthCheck(ctx context.Context, isStartup bool) (define.
 	if len(newCommand) < 1 || newCommand[0] == "" {
 		return define.HealthCheckNotDefined, "", fmt.Errorf("container %s has no defined healthcheck", c.ID())
 	}
-	rPipe, wPipe, err := os.Pipe()
-	if err != nil {
-		return define.HealthCheckInternalError, "", fmt.Errorf("unable to create pipe for healthcheck session: %w", err)
-	}
-	defer wPipe.Close()
-	defer rPipe.Close()
 
 	streams := new(define.AttachStreams)
+	output := &bytes.Buffer{}
 
 	streams.InputStream = bufio.NewReader(os.Stdin)
-	streams.OutputStream = wPipe
-	streams.ErrorStream = wPipe
+	streams.OutputStream = output
+	streams.ErrorStream = output
 	streams.AttachOutput = true
 	streams.AttachError = true
 	streams.AttachInput = true
 
-	stdout := []string{}
-	go func() {
-		scanner := bufio.NewScanner(rPipe)
-		for scanner.Scan() {
-			stdout = append(stdout, scanner.Text())
-		}
-	}()
-
 	logrus.Debugf("executing health check command %s for %s", strings.Join(newCommand, " "), c.ID())
 	timeStart := time.Now()
 	hcResult := define.HealthCheckSuccess
@@ -152,9 +135,9 @@ func (c *Container) runHealthCheck(ctx context.Context, isStartup bool) (define.
 		}
 	}
 
-	eventLog := strings.Join(stdout, "\n")
-	if len(eventLog) > MaxHealthCheckLogLength {
-		eventLog = eventLog[:MaxHealthCheckLogLength]
+	eventLog := output.String()
+	if c.config.HealthMaxLogSize != 0 && len(eventLog) > int(c.config.HealthMaxLogSize) {
+		eventLog = eventLog[:c.config.HealthMaxLogSize]
 	}
 
 	if timeEnd.Sub(timeStart) > c.HealthCheckConfig().Timeout {
@@ -164,19 +147,22 @@ func (c *Container) runHealthCheck(ctx context.Context, isStartup bool) (define.
 	}
 
 	hcl := newHealthCheckLog(timeStart, timeEnd, returnCode, eventLog)
-	logStatus, err := c.updateHealthCheckLog(hcl, inStartPeriod)
+
+	healthCheckResult, err := c.updateHealthCheckLog(hcl, inStartPeriod, isStartup)
 	if err != nil {
-		return hcResult, "", fmt.Errorf("unable to update health check log %s for %s: %w", c.healthCheckLogPath(), c.ID(), err)
+		return hcResult, "", fmt.Errorf("unable to update health check log %s for %s: %w", c.config.HealthLogDestination, c.ID(), err)
 	}
 
 	// Write HC event with appropriate status as the last thing before we
 	// return.
 	if hcResult == define.HealthCheckNotDefined || hcResult == define.HealthCheckInternalError {
-		return hcResult, logStatus, hcErr
+		return hcResult, healthCheckResult.Status, hcErr
+	}
+	if c.runtime.config.Engine.HealthcheckEvents {
+		c.newContainerHealthCheckEvent(healthCheckResult)
 	}
-	c.newContainerEvent(events.HealthStatus)
 
-	return hcResult, logStatus, hcErr
+	return hcResult, healthCheckResult.Status, hcErr
 }
 
 func (c *Container) processHealthCheckStatus(status string) error {
@@ -274,6 +260,7 @@ func (c *Container) incrementStartupHCSuccessCounter(ctx context.Context) {
 	if recreateTimer {
 		logrus.Infof("Startup healthcheck for container %s passed, recreating timer", c.ID())
 
+		oldUnit := c.state.HCUnitName
 		// Create the new, standard healthcheck timer first.
 		if err := c.createTimer(c.HealthCheckConfig().Interval.String(), false); err != nil {
 			logrus.Errorf("Error recreating container %s healthcheck: %v", c.ID(), err)
@@ -287,7 +274,14 @@ func (c *Container) incrementStartupHCSuccessCounter(ctx context.Context) {
 		// Which happens to be us.
 		// So this has to be last - after this, systemd serves us a
 		// SIGTERM and we exit.
-		if err := c.removeTransientFiles(ctx, true); err != nil {
+		// Special case, via SIGTERM we exit(1) which means systemd logs a failure in the unit.
+		// We do not want this as the unit will be leaked on failure states unless "reset-failed"
+		// is called. Fundamentally this is expected so switch it to exit 0.
+		// NOTE: This is only safe while being called from "podman healthcheck run" which we know
+		// is the case here as we should not alter the exit code of another process that just
+		// happened to call this.
+		shutdown.SetExitCode(0)
+		if err := c.removeTransientFiles(ctx, true, oldUnit); err != nil {
 			logrus.Errorf("Error removing container %s healthcheck: %v", c.ID(), err)
 			return
 		}
@@ -344,19 +338,15 @@ func newHealthCheckLog(start, end time.Time, exitCode int, log string) define.He
 	}
 }
 
-// updatedHealthCheckStatus updates the health status of the container
+// updateHealthStatus updates the health status of the container
 // in the healthcheck log
 func (c *Container) updateHealthStatus(status string) error {
-	healthCheck, err := c.getHealthCheckLog()
+	healthCheck, err := c.readHealthCheckLog()
 	if err != nil {
 		return err
 	}
 	healthCheck.Status = status
-	newResults, err := json.Marshal(healthCheck)
-	if err != nil {
-		return fmt.Errorf("unable to marshall healthchecks for writing status: %w", err)
-	}
-	return os.WriteFile(c.healthCheckLogPath(), newResults, 0700)
+	return c.writeHealthCheckLog(healthCheck)
 }
 
 // isUnhealthy returns true if the current health check status is unhealthy.
@@ -364,7 +354,7 @@ func (c *Container) isUnhealthy() (bool, error) {
 	if !c.HasHealthCheck() {
 		return false, nil
 	}
-	healthCheck, err := c.getHealthCheckLog()
+	healthCheck, err := c.readHealthCheckLog()
 	if err != nil {
 		return false, err
 	}
@@ -372,13 +362,20 @@ func (c *Container) isUnhealthy() (bool, error) {
 }
 
 // UpdateHealthCheckLog parses the health check results and writes the log
-func (c *Container) updateHealthCheckLog(hcl define.HealthCheckLog, inStartPeriod bool) (string, error) {
+func (c *Container) updateHealthCheckLog(hcl define.HealthCheckLog, inStartPeriod, isStartup bool) (define.HealthCheckResults, error) {
 	c.lock.Lock()
 	defer c.lock.Unlock()
 
-	healthCheck, err := c.getHealthCheckLog()
+	// If we are playing a kube yaml then let's honor the start period time for
+	// both failing and succeeding cases to match kube behavior.
+	// So don't update the health check log till the start period is over
+	if _, ok := c.config.Spec.Annotations[define.KubeHealthCheckAnnotation]; ok && inStartPeriod && !isStartup {
+		return define.HealthCheckResults{}, nil
+	}
+
+	healthCheck, err := c.readHealthCheckLog()
 	if err != nil {
-		return "", err
+		return define.HealthCheckResults{}, err
 	}
 	if hcl.ExitCode == 0 {
 		//	set status to healthy, reset failing state to 0
@@ -398,36 +395,57 @@ func (c *Container) updateHealthCheckLog(hcl define.HealthCheckLog, inStartPerio
 		}
 	}
 	healthCheck.Log = append(healthCheck.Log, hcl)
-	if len(healthCheck.Log) > MaxHealthCheckNumberLogs {
+	if c.config.HealthMaxLogCount != 0 && len(healthCheck.Log) > int(c.config.HealthMaxLogCount) {
 		healthCheck.Log = healthCheck.Log[1:]
 	}
-	newResults, err := json.Marshal(healthCheck)
+	return healthCheck, c.writeHealthCheckLog(healthCheck)
+}
+
+func (c *Container) witeToFileHealthCheckResults(path string, result define.HealthCheckResults) error {
+	newResults, err := json.Marshal(result)
 	if err != nil {
-		return "", fmt.Errorf("unable to marshall healthchecks for writing: %w", err)
+		return fmt.Errorf("unable to marshall healthchecks for writing: %w", err)
 	}
-	return healthCheck.Status, os.WriteFile(c.healthCheckLogPath(), newResults, 0700)
+	return os.WriteFile(path, newResults, 0700)
 }
 
-// HealthCheckLogPath returns the path for where the health check log is
-func (c *Container) healthCheckLogPath() string {
-	return filepath.Join(filepath.Dir(c.state.RunDir), "healthcheck.log")
+func (c *Container) getHealthCheckLogDestination() string {
+	var destination string
+	switch c.config.HealthLogDestination {
+	case define.DefaultHealthCheckLocalDestination, define.HealthCheckEventsLoggerDestination, "":
+		destination = filepath.Join(filepath.Dir(c.state.RunDir), "healthcheck.log")
+	default:
+		destination = filepath.Join(c.config.HealthLogDestination, c.ID()+"-healthcheck.log")
+	}
+	return destination
 }
 
-// getHealthCheckLog returns HealthCheck results by reading the container's
+func (c *Container) writeHealthCheckLog(result define.HealthCheckResults) error {
+	return c.witeToFileHealthCheckResults(c.getHealthCheckLogDestination(), result)
+}
+
+// readHealthCheckLog read HealthCheck logs from the path or events_logger
+// The caller should lock the container before this function is called.
+func (c *Container) readHealthCheckLog() (define.HealthCheckResults, error) {
+	return c.readFromFileHealthCheckLog(c.getHealthCheckLogDestination())
+}
+
+// readFromFileHealthCheckLog returns HealthCheck results by reading the container's
 // health check log file.  If the health check log file does not exist, then
 // an empty healthcheck struct is returned
 // The caller should lock the container before this function is called.
-func (c *Container) getHealthCheckLog() (define.HealthCheckResults, error) {
+func (c *Container) readFromFileHealthCheckLog(path string) (define.HealthCheckResults, error) {
 	var healthCheck define.HealthCheckResults
-	if _, err := os.Stat(c.healthCheckLogPath()); os.IsNotExist(err) {
-		return healthCheck, nil
-	}
-	b, err := os.ReadFile(c.healthCheckLogPath())
+	b, err := os.ReadFile(path)
 	if err != nil {
+		if errors.Is(err, fs.ErrNotExist) {
+			// If the file does not exists just return empty healthcheck and no error.
+			return healthCheck, nil
+		}
 		return healthCheck, fmt.Errorf("failed to read health check log file: %w", err)
 	}
 	if err := json.Unmarshal(b, &healthCheck); err != nil {
-		return healthCheck, fmt.Errorf("failed to unmarshal existing healthcheck results in %s: %w", c.healthCheckLogPath(), err)
+		return healthCheck, fmt.Errorf("failed to unmarshal existing healthcheck results in %s: %w", path, err)
 	}
 	return healthCheck, nil
 }
@@ -453,7 +471,7 @@ func (c *Container) healthCheckStatus() (string, error) {
 		return "", err
 	}
 
-	results, err := c.getHealthCheckLog()
+	results, err := c.readHealthCheckLog()
 	if err != nil {
 		return "", fmt.Errorf("unable to get healthcheck log for %s: %w", c.ID(), err)
 	}
