@@ -1,3 +1,5 @@
+//go:build !remote
+
 package compat
 
 import (
@@ -11,6 +13,7 @@ import (
 	"path/filepath"
 	"strconv"
 	"strings"
+	"syscall"
 	"time"
 
 	"github.com/containers/buildah"
@@ -33,6 +36,13 @@ import (
 	"github.com/sirupsen/logrus"
 )
 
+func genSpaceErr(err error) error {
+	if errors.Is(err, syscall.ENOSPC) {
+		return fmt.Errorf("context directory may be too large: %w", err)
+	}
+	return err
+}
+
 func BuildImage(w http.ResponseWriter, r *http.Request) {
 	if hdr, found := r.Header["Content-Type"]; found && len(hdr) > 0 {
 		contentType := hdr[0]
@@ -72,6 +82,13 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 	}()
 
 	contextDirectory, err := extractTarFile(anchorDir, r)
+	if err != nil {
+		utils.InternalServerError(w, genSpaceErr(err))
+		return
+	}
+
+	runtime := r.Context().Value(api.RuntimeKey).(*libpod.Runtime)
+	conf, err := runtime.GetConfigNoCopy()
 	if err != nil {
 		utils.InternalServerError(w, err)
 		return
@@ -89,6 +106,7 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 		CacheTo                 string   `schema:"cacheto"`
 		CacheTTL                string   `schema:"cachettl"`
 		CgroupParent            string   `schema:"cgroupparent"`
+		CompatVolumes           bool     `schema:"compatvolumes"`
 		Compression             uint64   `schema:"compression"`
 		ConfigureNetwork        string   `schema:"networkmode"`
 		CPPFlags                string   `schema:"cppflags"`
@@ -136,6 +154,8 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 		Rm                      bool     `schema:"rm"`
 		RusageLogFile           string   `schema:"rusagelogfile"`
 		Remote                  string   `schema:"remote"`
+		Retry                   int      `schema:"retry"`
+		RetryDelay              string   `schema:"retry-delay"`
 		Seccomp                 string   `schema:"seccomp"`
 		Secrets                 string   `schema:"secrets"`
 		SecurityOpt             string   `schema:"securityopt"`
@@ -158,6 +178,8 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 		ShmSize:          64 * 1024 * 1024,
 		TLSVerify:        true,
 		SkipUnusedStages: true,
+		Retry:            int(conf.Engine.Retry),
+		RetryDelay:       conf.Engine.RetryDelay,
 	}
 
 	decoder := utils.GetDecoder(r)
@@ -202,7 +224,7 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 		}
 		tempDir, subDir, err := buildahDefine.TempDirForURL(anchorDir, "buildah", query.Remote)
 		if err != nil {
-			utils.InternalServerError(w, err)
+			utils.InternalServerError(w, genSpaceErr(err))
 			return
 		}
 		if tempDir != "" {
@@ -632,7 +654,10 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 		AuthFilePath:     authfile,
 		DockerAuthConfig: creds,
 	}
-	utils.PossiblyEnforceDockerHub(r, systemContext)
+	if err := utils.PossiblyEnforceDockerHub(r, systemContext); err != nil {
+		utils.Error(w, http.StatusInternalServerError, fmt.Errorf("checking to enforce DockerHub: %w", err))
+		return
+	}
 
 	if _, found := r.URL.Query()["tlsVerify"]; found {
 		systemContext.DockerInsecureSkipTLSVerify = types.NewOptionalBool(!query.TLSVerify)
@@ -658,7 +683,15 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 		return
 	}
 
-	runtime := r.Context().Value(api.RuntimeKey).(*libpod.Runtime)
+	retryDelay := 2 * time.Second
+	if query.RetryDelay != "" {
+		retryDelay, err = time.ParseDuration(query.RetryDelay)
+		if err != nil {
+			utils.BadRequest(w, "retry-delay", query.RetryDelay, err)
+			return
+		}
+	}
+
 	buildOptions := buildahDefine.BuildOptions{
 		AddCapabilities:         addCaps,
 		AdditionalBuildContexts: additionalBuildContexts,
@@ -694,6 +727,7 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 			Secrets:            secrets,
 			Volumes:            query.Volumes,
 		},
+		CompatVolumes:                  types.NewOptionalBool(query.CompatVolumes),
 		Compression:                    compression,
 		ConfigureNetwork:               parseNetworkConfigurationPolicy(query.ConfigureNetwork),
 		ContextDirectory:               contextDirectory,
@@ -715,7 +749,7 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 		Layers:                         query.Layers,
 		LogRusage:                      query.LogRusage,
 		Manifest:                       query.Manifest,
-		MaxPullPushRetries:             3,
+		MaxPullPushRetries:             query.Retry,
 		NamespaceOptions:               nsoptions,
 		NoCache:                        query.NoCache,
 		OSFeatures:                     query.OSFeatures,
@@ -724,7 +758,7 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 		Output:                         output,
 		OutputFormat:                   format,
 		PullPolicy:                     pullPolicy,
-		PullPushRetryDelay:             2 * time.Second,
+		PullPushRetryDelay:             retryDelay,
 		Quiet:                          query.Quiet,
 		Registry:                       registry,
 		RemoveIntermediateCtrs:         query.Rm,
@@ -738,7 +772,12 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 		UnsetLabels:                    query.UnsetLabels,
 	}
 
-	for _, platformSpec := range query.Platform {
+	platforms := query.Platform
+	if len(platforms) == 1 {
+		// Docker API uses comma separated platform arg so match this here
+		platforms = strings.Split(query.Platform[0], ",")
+	}
+	for _, platformSpec := range platforms {
 		os, arch, variant, err := parse.Platform(platformSpec)
 		if err != nil {
 			utils.BadRequest(w, "platform", platformSpec, err)
