@@ -1,3 +1,5 @@
+//go:build !remote
+
 package compat
 
 import (
@@ -58,7 +60,8 @@ func GetDiskUsage(w http.ResponseWriter, r *http.Request) {
 			State:      o.Status,
 			Status:     o.Status,
 			HostConfig: struct {
-				NetworkMode string `json:",omitempty"`
+				NetworkMode string            `json:",omitempty"`
+				Annotations map[string]string `json:",omitempty"`
 			}{},
 			NetworkSettings: nil,
 			Mounts:          nil,
