@@ -14,9 +14,10 @@ type EventsOptions struct {
 //
 //go:generate go run ../generator/generator.go PruneOptions
 type PruneOptions struct {
-	All     *bool
-	Filters map[string][]string
-	Volumes *bool
+	All      *bool
+	Filters  map[string][]string
+	Volumes  *bool
+	External *bool
 }
 
 // VersionOptions are optional options for getting version info
@@ -37,3 +38,13 @@ type DiskOptions struct {
 //go:generate go run ../generator/generator.go InfoOptions
 type InfoOptions struct {
 }
+
+// CheckOptions are optional options for storage consistency check/repair
+//
+//go:generate go run ../generator/generator.go CheckOptions
+type CheckOptions struct {
+	Quick                       *bool   `schema:"quick"`
+	Repair                      *bool   `schema:"repair"`
+	RepairLossy                 *bool   `schema:"repair_lossy"`
+	UnreferencedLayerMaximumAge *string `schema:"unreferenced_layer_max_age"`
+}
