@@ -3,12 +3,11 @@ package entities
 import (
 	"errors"
 	"strings"
-	"time"
 
 	commonFlag "github.com/containers/common/pkg/flag"
-	"github.com/containers/podman/v4/libpod/define"
-	"github.com/containers/podman/v4/pkg/specgen"
-	"github.com/containers/podman/v4/pkg/util"
+	"github.com/containers/podman/v5/pkg/domain/entities/types"
+	"github.com/containers/podman/v5/pkg/specgen"
+	"github.com/containers/podman/v5/pkg/util"
 	"github.com/opencontainers/runtime-spec/specs-go"
 )
 
@@ -18,50 +17,25 @@ type PodKillOptions struct {
 	Signal string
 }
 
-type PodKillReport struct {
-	Errs []error
-	Id   string //nolint:revive,stylecheck
-}
+type PodKillReport = types.PodKillReport
 
-type ListPodsReport struct {
-	Cgroup     string
-	Containers []*ListPodContainer
-	Created    time.Time
-	Id         string //nolint:revive,stylecheck
-	InfraId    string //nolint:revive,stylecheck
-	Name       string
-	Namespace  string
-	// Network names connected to infra container
-	Networks []string
-	Status   string
-	Labels   map[string]string
-}
+type ListPodsReport = types.ListPodsReport
 
-type ListPodContainer struct {
-	Id     string //nolint:revive,stylecheck
-	Names  string
-	Status string
-}
+type ListPodContainer = types.ListPodContainer
 
 type PodPauseOptions struct {
 	All    bool
 	Latest bool
 }
 
-type PodPauseReport struct {
-	Errs []error
-	Id   string //nolint:revive,stylecheck
-}
+type PodPauseReport = types.PodPauseReport
 
 type PodunpauseOptions struct {
 	All    bool
 	Latest bool
 }
 
-type PodUnpauseReport struct {
-	Errs []error
-	Id   string //nolint:revive,stylecheck
-}
+type PodUnpauseReport = types.PodUnpauseReport
 
 type PodStopOptions struct {
 	All     bool
@@ -70,30 +44,20 @@ type PodStopOptions struct {
 	Timeout int
 }
 
-type PodStopReport struct {
-	Errs []error
-	Id   string //nolint:revive,stylecheck
-}
+type PodStopReport = types.PodStopReport
 
 type PodRestartOptions struct {
 	All    bool
 	Latest bool
 }
 
-type PodRestartReport struct {
-	Errs []error
-	Id   string //nolint:revive,stylecheck
-}
-
+type PodRestartReport = types.PodRestartReport
 type PodStartOptions struct {
 	All    bool
 	Latest bool
 }
 
-type PodStartReport struct {
-	Errs []error
-	Id   string //nolint:revive,stylecheck
-}
+type PodStartReport = types.PodStartReport
 
 type PodRmOptions struct {
 	All     bool
@@ -103,16 +67,9 @@ type PodRmOptions struct {
 	Timeout *uint
 }
 
-type PodRmReport struct {
-	Err error
-	Id  string //nolint:revive,stylecheck
-}
+type PodRmReport = types.PodRmReport
 
-// PddSpec is an abstracted version of PodSpecGen designed to eventually accept options
-// not meant to be in a specgen
-type PodSpec struct {
-	PodSpecGen specgen.PodSpecGenerator
-}
+type PodSpec = types.PodSpec
 
 // PodCreateOptions provides all possible options for creating a pod and its infra container.
 // The JSON tags below are made to match the respective field in ContainerCreateOptions for the purpose of mapping.
@@ -135,6 +92,7 @@ type PodCreateOptions struct {
 	Net                *NetOptions       `json:"net,omitempty"`
 	Share              []string          `json:"share,omitempty"`
 	ShareParent        *bool             `json:"share_parent,omitempty"`
+	Restart            string            `json:"restart,omitempty"`
 	Pid                string            `json:"pid,omitempty"`
 	Cpus               float64           `json:"cpus,omitempty"`
 	CpusetCpus         string            `json:"cpuset_cpus,omitempty"`
@@ -143,6 +101,7 @@ type PodCreateOptions struct {
 	VolumesFrom        []string          `json:"volumes_from,omitempty"`
 	SecurityOpt        []string          `json:"security_opt,omitempty"`
 	Sysctl             []string          `json:"sysctl,omitempty"`
+	Uts                string            `json:"uts,omitempty"`
 }
 
 // PodLogsOptions describes the options to extract pod logs.
@@ -175,130 +134,138 @@ const (
 )
 
 type ContainerCreateOptions struct {
-	Annotation         []string
-	Attach             []string
-	Authfile           string
-	BlkIOWeight        string
-	BlkIOWeightDevice  []string
-	CapAdd             []string
-	CapDrop            []string
-	CgroupNS           string
-	CgroupsMode        string
-	CgroupParent       string `json:"cgroup_parent,omitempty"`
-	CIDFile            string
-	ConmonPIDFile      string `json:"container_conmon_pidfile,omitempty"`
-	CPUPeriod          uint64
-	CPUQuota           int64
-	CPURTPeriod        uint64
-	CPURTRuntime       int64
-	CPUShares          uint64
-	CPUS               float64 `json:"cpus,omitempty"`
-	CPUSetCPUs         string  `json:"cpuset_cpus,omitempty"`
-	CPUSetMems         string
-	Devices            []string `json:"devices,omitempty"`
-	DeviceCgroupRule   []string
-	DeviceReadBPs      []string `json:"device_read_bps,omitempty"`
-	DeviceReadIOPs     []string
-	DeviceWriteBPs     []string
-	DeviceWriteIOPs    []string
-	Entrypoint         *string `json:"container_command,omitempty"`
-	Env                []string
-	EnvHost            bool
-	EnvFile            []string
-	Expose             []string
-	GIDMap             []string
-	GroupAdd           []string
-	HealthCmd          string
-	HealthInterval     string
-	HealthRetries      uint
-	HealthStartPeriod  string
-	HealthTimeout      string
-	HealthOnFailure    string
-	Hostname           string `json:"hostname,omitempty"`
-	HTTPProxy          bool
-	HostUsers          []string
-	ImageVolume        string
-	Init               bool
-	InitContainerType  string
-	InitPath           string
-	Interactive        bool
-	IPC                string
-	Label              []string
-	LabelFile          []string
-	LogDriver          string
-	LogOptions         []string
-	Memory             string
-	MemoryReservation  string
-	MemorySwap         string
-	MemorySwappiness   int64
-	Name               string `json:"container_name"`
-	NoHealthCheck      bool
-	OOMKillDisable     bool
-	OOMScoreAdj        *int
-	Arch               string
-	OS                 string
-	Variant            string
-	PID                string `json:"pid,omitempty"`
-	PIDsLimit          *int64
-	Platform           string
-	Pod                string
-	PodIDFile          string
-	Personality        string
-	PreserveFDs        uint
-	Privileged         bool
-	PublishAll         bool
-	Pull               string
-	Quiet              bool
-	ReadOnly           bool
-	ReadWriteTmpFS     bool
-	Restart            string
-	Replace            bool
-	Requires           []string
-	Rm                 bool
-	RootFS             bool
-	Secrets            []string
-	SecurityOpt        []string `json:"security_opt,omitempty"`
-	SdNotifyMode       string
-	ShmSize            string
-	ShmSizeSystemd     string
-	SignaturePolicy    string
-	StartupHCCmd       string
-	StartupHCInterval  string
-	StartupHCRetries   uint
-	StartupHCSuccesses uint
-	StartupHCTimeout   string
-	StopSignal         string
-	StopTimeout        uint
-	StorageOpts        []string
-	SubGIDName         string
-	SubUIDName         string
-	Sysctl             []string `json:"sysctl,omitempty"`
-	Systemd            string
-	Timeout            uint
-	TLSVerify          commonFlag.OptionalBool
-	TmpFS              []string
-	TTY                bool
-	Timezone           string
-	Umask              string
-	EnvMerge           []string
-	UnsetEnv           []string
-	UnsetEnvAll        bool
-	UIDMap             []string
-	Ulimit             []string
-	User               string
-	UserNS             string `json:"-"`
-	UTS                string
-	Mount              []string
-	Volume             []string `json:"volume,omitempty"`
-	VolumesFrom        []string `json:"volumes_from,omitempty"`
-	Workdir            string
-	SeccompPolicy      string
-	PidFile            string
-	ChrootDirs         []string
-	IsInfra            bool
-	IsClone            bool
-	DecryptionKeys     []string
-	Net                *NetOptions `json:"net,omitempty"`
+	Annotation           []string
+	Attach               []string
+	Authfile             string
+	BlkIOWeight          string
+	BlkIOWeightDevice    []string
+	CapAdd               []string
+	CapDrop              []string
+	CgroupNS             string
+	CgroupsMode          string
+	CgroupParent         string `json:"cgroup_parent,omitempty"`
+	CIDFile              string
+	ConmonPIDFile        string `json:"container_conmon_pidfile,omitempty"`
+	CPUPeriod            uint64
+	CPUQuota             int64
+	CPURTPeriod          uint64
+	CPURTRuntime         int64
+	CPUShares            uint64
+	CPUS                 float64 `json:"cpus,omitempty"`
+	CPUSetCPUs           string  `json:"cpuset_cpus,omitempty"`
+	CPUSetMems           string
+	Devices              []string `json:"devices,omitempty"`
+	DeviceCgroupRule     []string
+	DeviceReadBPs        []string `json:"device_read_bps,omitempty"`
+	DeviceReadIOPs       []string
+	DeviceWriteBPs       []string
+	DeviceWriteIOPs      []string
+	Entrypoint           *string `json:"container_command,omitempty"`
+	Env                  []string
+	EnvHost              bool
+	EnvFile              []string
+	Expose               []string
+	GIDMap               []string
+	GPUs                 []string
+	GroupAdd             []string
+	HealthCmd            string
+	HealthInterval       string
+	HealthRetries        uint
+	HealthLogDestination string
+	HealthMaxLogCount    uint
+	HealthMaxLogSize     uint
+	HealthStartPeriod    string
+	HealthTimeout        string
+	HealthOnFailure      string
+	Hostname             string `json:"hostname,omitempty"`
+	HTTPProxy            bool
+	HostUsers            []string
+	ImageVolume          string
+	Init                 bool
+	InitContainerType    string
+	InitPath             string
+	IntelRdtClosID       string
+	Interactive          bool
+	IPC                  string
+	Label                []string
+	LabelFile            []string
+	LogDriver            string
+	LogOptions           []string
+	Memory               string
+	MemoryReservation    string
+	MemorySwap           string
+	MemorySwappiness     int64
+	Name                 string `json:"container_name"`
+	NoHealthCheck        bool
+	OOMKillDisable       bool
+	OOMScoreAdj          *int
+	Arch                 string
+	OS                   string
+	Variant              string
+	PID                  string `json:"pid,omitempty"`
+	PIDsLimit            *int64
+	Platform             string
+	Pod                  string
+	PodIDFile            string
+	Personality          string
+	PreserveFDs          uint
+	PreserveFD           []uint
+	Privileged           bool
+	PublishAll           bool
+	Pull                 string
+	Quiet                bool
+	ReadOnly             bool
+	ReadWriteTmpFS       bool
+	Restart              string
+	Replace              bool
+	Requires             []string
+	Retry                *uint  `json:"retry,omitempty"`
+	RetryDelay           string `json:"retry_delay,omitempty"`
+	Rm                   bool
+	RootFS               bool
+	Secrets              []string
+	SecurityOpt          []string `json:"security_opt,omitempty"`
+	SdNotifyMode         string
+	ShmSize              string
+	ShmSizeSystemd       string
+	SignaturePolicy      string
+	StartupHCCmd         string
+	StartupHCInterval    string
+	StartupHCRetries     uint
+	StartupHCSuccesses   uint
+	StartupHCTimeout     string
+	StopSignal           string
+	StopTimeout          uint
+	StorageOpts          []string
+	SubGIDName           string
+	SubUIDName           string
+	Sysctl               []string `json:"sysctl,omitempty"`
+	Systemd              string
+	Timeout              uint
+	TLSVerify            commonFlag.OptionalBool
+	TmpFS                []string
+	TTY                  bool
+	Timezone             string
+	Umask                string
+	EnvMerge             []string
+	UnsetEnv             []string
+	UnsetEnvAll          bool
+	UIDMap               []string
+	Ulimit               []string
+	User                 string
+	UserNS               string `json:"-"`
+	UTS                  string
+	Mount                []string
+	Volume               []string `json:"volume,omitempty"`
+	VolumesFrom          []string `json:"volumes_from,omitempty"`
+	Workdir              string
+	SeccompPolicy        string
+	PidFile              string
+	ChrootDirs           []string
+	IsInfra              bool
+	IsClone              bool
+	DecryptionKeys       []string
+	Net                  *NetOptions `json:"net,omitempty"`
 
 	CgroupConf []string
 
@@ -309,19 +276,15 @@ type ContainerCreateOptions struct {
 func NewInfraContainerCreateOptions() ContainerCreateOptions {
 	options := ContainerCreateOptions{
 		IsInfra:          true,
-		ImageVolume:      "bind",
+		ImageVolume:      "anonymous",
 		MemorySwappiness: -1,
 	}
 	return options
 }
 
-type PodCreateReport struct {
-	Id string //nolint:revive,stylecheck
-}
+type PodCreateReport = types.PodCreateReport
 
-type PodCloneReport struct {
-	Id string //nolint:revive,stylecheck
-}
+type PodCloneReport = types.PodCloneReport
 
 func (p *PodCreateOptions) CPULimits() *specs.LinuxCPU {
 	cpu := &specs.LinuxCPU{}
@@ -358,6 +321,12 @@ func ToPodSpecGen(s specgen.PodSpecGenerator, p *PodCreateOptions) (*specgen.Pod
 		return nil, err
 	}
 	s.Ipc = out
+
+	out, err = specgen.ParseNamespace(p.Uts)
+	if err != nil {
+		return nil, err
+	}
+	s.UtsNs = out
 	s.Hostname = p.Hostname
 	s.ExitPolicy = p.ExitPolicy
 	s.Labels = p.Labels
@@ -375,6 +344,14 @@ func ToPodSpecGen(s specgen.PodSpecGenerator, p *PodCreateOptions) (*specgen.Pod
 	s.ShareParent = p.ShareParent
 	s.PodCreateCommand = p.CreateCommand
 	s.VolumesFrom = p.VolumesFrom
+	if p.Restart != "" {
+		policy, retries, err := util.ParseRestartPolicy(p.Restart)
+		if err != nil {
+			return nil, err
+		}
+		s.RestartPolicy = policy
+		s.RestartRetries = &retries
+	}
 
 	// Networking config
 
@@ -426,10 +403,7 @@ type PodPruneOptions struct {
 	Force bool `json:"force" schema:"force"`
 }
 
-type PodPruneReport struct {
-	Err error
-	Id  string //nolint:revive,stylecheck
-}
+type PodPruneReport = types.PodPruneReport
 
 type PodTopOptions struct {
 	// CLI flags.
@@ -453,9 +427,7 @@ type PodPSOptions struct {
 	Sort      string
 }
 
-type PodInspectReport struct {
-	*define.InspectPodData
-}
+type PodInspectReport = types.PodInspectReport
 
 // PodStatsOptions are options for the pod stats command.
 type PodStatsOptions struct {
@@ -466,35 +438,7 @@ type PodStatsOptions struct {
 }
 
 // PodStatsReport includes pod-resource statistics data.
-type PodStatsReport struct {
-	// Percentage of CPU utilized by pod
-	// example: 75.5%
-	CPU string
-	// Humanized Memory usage and maximum
-	// example: 12mb / 24mb
-	MemUsage string
-	// Memory usage and maximum in bytes
-	// example: 1,000,000 / 4,000,000
-	MemUsageBytes string
-	// Percentage of Memory utilized by pod
-	// example: 50.5%
-	Mem string
-	// Network usage inbound + outbound
-	NetIO string
-	// Humanized disk usage read + write
-	BlockIO string
-	// Container PID
-	PIDS string
-	// Pod ID
-	// example: 62310217a19e
-	Pod string
-	// Container ID
-	// example: e43534f89a7d
-	CID string
-	// Pod Name
-	// example: elastic_pascal
-	Name string
-}
+type PodStatsReport = types.PodStatsReport
 
 // ValidatePodStatsOptions validates the specified slice and options. Allows
 // for sharing code in the front- and the back-end.
@@ -525,7 +469,7 @@ func ValidatePodStatsOptions(args []string, options *PodStatsOptions) error {
 // PodLogsOptionsToContainerLogsOptions converts PodLogOptions to ContainerLogOptions
 func PodLogsOptionsToContainerLogsOptions(options PodLogsOptions) ContainerLogsOptions {
 	// PodLogsOptions are similar but contains few extra fields like ctrName
-	// So cast other values as is so we can re-use the code
+	// So cast other values as is so we can reuse the code
 	containerLogsOpts := ContainerLogsOptions{
 		Details:      options.Details,
 		Latest:       options.Latest,
