@@ -60,8 +60,6 @@ var _ = Describe("run basic podman commands", func() {
 	})
 
 	It("Volume ops", func() {
-		skipIfVmtype(define.HyperVVirt, "FIXME: #21036 - Hyper-V podman run -v fails due to path translation issues")
-
 		tDir, err := filepath.Abs(GinkgoT().TempDir())
 		Expect(err).ToNot(HaveOccurred())
 		roFile := filepath.Join(tDir, "attr-test-file")
@@ -88,6 +86,14 @@ var _ = Describe("run basic podman commands", func() {
 		runAlp, err := mb.setCmd(bm.withPodmanCommand([]string{"run", "-v", tDir + ":/test:Z", "quay.io/libpod/alpine_nginx", "ls", "/test/attr-test-file"})).run()
 		Expect(err).ToNot(HaveOccurred())
 		Expect(runAlp).To(Exit(0))
+
+		// Test build with --volume option
+		cf := filepath.Join(tDir, "Containerfile")
+		err = os.WriteFile(cf, []byte("FROM quay.io/libpod/alpine_nginx\nRUN ls /test/attr-test-file\n"), 0o644)
+		Expect(err).ToNot(HaveOccurred())
+		build, err := mb.setCmd(bm.withPodmanCommand([]string{"build", "-t", name, "-v", tDir + ":/test", tDir})).run()
+		Expect(err).ToNot(HaveOccurred())
+		Expect(build).To(Exit(0))
 	})
 
 	It("Volume should be virtiofs", func() {
@@ -207,6 +213,46 @@ var _ = Describe("run basic podman commands", func() {
 		Expect(ls).To(Exit(0))
 		Expect(ls.outputToString()).To(ContainSubstring(testString))
 	})
+
+	It("podman build contexts", func() {
+		skipIfVmtype(define.HyperVVirt, "FIXME: #23429 - Error running podman build with option --build-context on Hyper-V")
+		skipIfVmtype(define.QemuVirt, "FIXME: #23433 - Additional build contexts should be sent as additional tar files")
+		name := randomString()
+		i := new(initMachine)
+		session, err := mb.setName(name).setCmd(i.withImage(mb.imagePath).withNow()).run()
+		Expect(err).ToNot(HaveOccurred())
+		Expect(session).To(Exit(0))
+
+		mainContextDir := GinkgoT().TempDir()
+		cfile := filepath.Join(mainContextDir, "test1")
+		err = os.WriteFile(cfile, []byte(name), 0o644)
+		Expect(err).ToNot(HaveOccurred())
+
+		additionalContextDir := GinkgoT().TempDir()
+		cfile = filepath.Join(additionalContextDir, "test2")
+		err = os.WriteFile(cfile, []byte(name), 0o644)
+		Expect(err).ToNot(HaveOccurred())
+
+		cfile = filepath.Join(mainContextDir, "Containerfile")
+		err = os.WriteFile(cfile, []byte("FROM quay.io/libpod/alpine_nginx\nCOPY test1 /\nCOPY --from=test-context test2 /\n"), 0o644)
+		Expect(err).ToNot(HaveOccurred())
+
+		bm := basicMachine{}
+		build, err := mb.setCmd(bm.withPodmanCommand([]string{"build", "-t", name, "--build-context", "test-context=" + additionalContextDir, mainContextDir})).run()
+		Expect(err).ToNot(HaveOccurred())
+		Expect(build).To(Exit(0))
+		Expect(build.outputToString()).To(ContainSubstring("COMMIT"))
+
+		run, err := mb.setCmd(bm.withPodmanCommand([]string{"run", name, "cat", "/test1"})).run()
+		Expect(err).ToNot(HaveOccurred())
+		Expect(run).To(Exit(0))
+		Expect(build.outputToString()).To(ContainSubstring(name))
+
+		run, err = mb.setCmd(bm.withPodmanCommand([]string{"run", name, "cat", "/test2"})).run()
+		Expect(err).ToNot(HaveOccurred())
+		Expect(run).To(Exit(0))
+		Expect(build.outputToString()).To(ContainSubstring(name))
+	})
 })
 
 func testHTTPServer(port string, shouldErr bool, expectedResponse string) {
