@@ -1,14 +1,18 @@
 package e2e_test
 
 import (
+	"cmp"
+	"errors"
 	"fmt"
-	"io"
 	"os"
 	"path/filepath"
 	"runtime"
+	"slices"
+	"strings"
 	"testing"
+	"time"
 
-	"github.com/containers/podman/v5/pkg/machine/compression"
+	"github.com/containers/common/pkg/config"
 	"github.com/containers/podman/v5/pkg/machine/define"
 	"github.com/containers/podman/v5/pkg/machine/provider"
 	"github.com/containers/podman/v5/pkg/machine/vmconfigs"
@@ -33,7 +37,11 @@ var (
 
 func init() {
 	if value, ok := os.LookupEnv("TMPDIR"); ok {
-		tmpDir = value
+		var err error
+		tmpDir, err = setTmpDir(value)
+		if err != nil {
+			fmt.Printf("failed to set TMPDIR: %q\n", err)
+		}
 	}
 }
 
@@ -54,21 +62,65 @@ var _ = BeforeSuite(func() {
 	if err != nil {
 		Fail("unable to create testProvider")
 	}
-	if testProvider.VMType() == define.WSLVirt {
-		pullError = pullWSLDisk()
-	} else {
-		pullError = pullOCITestDisk(tmpDir, testProvider.VMType())
+
+	testDiskProvider := testProvider.VMType()
+	if testDiskProvider == define.LibKrun {
+		testDiskProvider = define.AppleHvVirt // libkrun uses the applehv image for testing
 	}
+	pullError = pullOCITestDisk(tmpDir, testDiskProvider)
+
 	if pullError != nil {
-		Fail(fmt.Sprintf("failed to pull wsl disk: %q", pullError))
+		Fail(fmt.Sprintf("failed to pull disk: %q", pullError))
 	}
 })
 
-var _ = SynchronizedAfterSuite(func() {}, func() {})
+type timing struct {
+	name   string
+	length time.Duration
+}
+
+var timings []timing
+
+var _ = AfterEach(func() {
+	r := CurrentSpecReport()
+	timings = append(timings, timing{
+		name:   r.FullText(),
+		length: r.RunTime,
+	})
+})
+
+var _ = SynchronizedAfterSuite(func() {}, func() {
+	slices.SortFunc(timings, func(a, b timing) int {
+		return cmp.Compare(a.length, b.length)
+	})
+	for _, t := range timings {
+		GinkgoWriter.Printf("%s\t\t%f seconds\n", t.name, t.length.Seconds())
+	}
+})
+
+// The config does not matter to much for our testing, however we
+// would like to be sure podman machine is not effected by certain
+// settings as we should be using full URLs anywhere.
+// https://github.com/containers/podman/issues/24567
+const sshConfigContent = `
+Host *
+  User NOT_REAL
+  Port 9999
+Host 127.0.0.1
+  User blah
+  IdentityFile ~/.ssh/id_ed25519
+`
 
 func setup() (string, *machineTestBuilder) {
 	// Set TMPDIR if this needs a new directory
-	homeDir, err := os.MkdirTemp("", "podman_test")
+	if value, ok := os.LookupEnv("TMPDIR"); ok {
+		var err error
+		tmpDir, err = setTmpDir(value)
+		if err != nil {
+			Fail(fmt.Sprintf("failed to set TMPDIR: %q", err))
+		}
+	}
+	homeDir, err := os.MkdirTemp(tmpDir, "podman_test")
 	if err != nil {
 		Fail(fmt.Sprintf("failed to create home directory: %q", err))
 	}
@@ -79,7 +131,7 @@ func setup() (string, *machineTestBuilder) {
 	if err != nil {
 		Fail(fmt.Sprintf("failed to create ssh config: %q", err))
 	}
-	if _, err := sshConfig.WriteString("IdentitiesOnly=yes"); err != nil {
+	if _, err := sshConfig.WriteString(sshConfigContent); err != nil {
 		Fail(fmt.Sprintf("failed to write ssh config: %q", err))
 	}
 	if err := sshConfig.Close(); err != nil {
@@ -99,6 +151,9 @@ func setup() (string, *machineTestBuilder) {
 	if err := os.Unsetenv("SSH_AUTH_SOCK"); err != nil {
 		Fail("unable to unset SSH_AUTH_SOCK")
 	}
+	if err := os.Setenv("PODMAN_CONNECTIONS_CONF", filepath.Join(homeDir, "connections.json")); err != nil {
+		Fail("failed to set PODMAN_CONNECTIONS_CONF")
+	}
 	mb, err := newMB()
 	if err != nil {
 		Fail(fmt.Sprintf("failed to create machine test: %q", err))
@@ -112,37 +167,11 @@ func setup() (string, *machineTestBuilder) {
 			Fail(fmt.Sprintf("failed to close src reader %q: %q", src.Name(), err))
 		}
 	}()
-	mb.imagePath = filepath.Join(homeDir, suiteImageName)
-	dest, err := os.Create(mb.imagePath)
-	if err != nil {
-		Fail(fmt.Sprintf("failed to create file %s: %q", mb.imagePath, err))
-	}
-	defer func() {
-		if err := dest.Close(); err != nil {
-			Fail(fmt.Sprintf("failed to close destination file %q: %q\n", dest.Name(), err))
-		}
-	}()
-	fmt.Printf("--> copying %q to %q\n", src.Name(), dest.Name())
-	if runtime.GOOS != "darwin" {
-		if _, err := io.Copy(dest, src); err != nil {
-			Fail(fmt.Sprintf("failed to copy %ss to %s: %q", fqImageName, mb.imagePath, err))
-		}
-	} else {
-		if err := copySparse(dest, src); err != nil {
-			Fail(fmt.Sprintf("failed to copy %q to %q: %q", src.Name(), dest.Name(), err))
-		}
-	}
+	mb.imagePath = fqImageName
 	return homeDir, mb
 }
 
-func teardown(origHomeDir string, testDir string, mb *machineTestBuilder) {
-	r := new(rmMachine)
-	for _, name := range mb.names {
-		if _, err := mb.setName(name).setCmd(r.withForce()).run(); err != nil {
-			GinkgoWriter.Printf("error occurred rm'ing machine: %q\n", err)
-		}
-	}
-
+func teardown(origHomeDir string, testDir string) {
 	if err := utils.GuardedRemoveAll(testDir); err != nil {
 		Fail(fmt.Sprintf("failed to remove test dir: %q", err))
 	}
@@ -157,14 +186,45 @@ func teardown(origHomeDir string, testDir string, mb *machineTestBuilder) {
 	}
 }
 
-// copySparse is a helper method for tests only; caller is responsible for closures
-func copySparse(dst io.WriteSeeker, src io.Reader) (retErr error) {
-	spWriter := compression.NewSparseWriter(dst)
-	defer func() {
-		if err := spWriter.Close(); err != nil && retErr == nil {
-			retErr = err
+var (
+	mb      *machineTestBuilder
+	testDir string
+)
+
+var _ = BeforeEach(func() {
+	testDir, mb = setup()
+	DeferCleanup(func() {
+		teardown(originalHomeDir, testDir)
+	})
+})
+
+func setTmpDir(value string) (string, error) {
+	switch {
+	case runtime.GOOS != "darwin":
+		tmpDir = value
+	case len(value) >= 22:
+		return "", errors.New(value + " path length should be less than 22 characters")
+	case value == "":
+		return "", errors.New("TMPDIR cannot be empty. Set to directory mounted on podman machine (e.g. /private/tmp)")
+	default:
+		cfg, err := config.Default()
+		if err != nil {
+			return "", err
 		}
-	}()
-	_, err := io.Copy(spWriter, src)
-	return err
+		volumes := cfg.Machine.Volumes.Get()
+		containsPath := false
+		for _, volume := range volumes {
+			parts := strings.Split(volume, ":")
+			hostPath := parts[0]
+			if strings.Contains(value, hostPath) {
+				containsPath = true
+				break
+			}
+		}
+		if !containsPath {
+			return "", fmt.Errorf("%s cannot be used. Change to directory mounted on podman machine (e.g. /private/tmp)", value)
+		}
+		tmpDir = value
+	}
+	return tmpDir, nil
 }
