@@ -1,23 +1,13 @@
 package e2e_test
 
 import (
+	"github.com/containers/podman/v5/pkg/machine/define"
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
 	. "github.com/onsi/gomega/gexec"
 )
 
 var _ = Describe("podman machine ssh", func() {
-	var (
-		mb      *machineTestBuilder
-		testDir string
-	)
-
-	BeforeEach(func() {
-		testDir, mb = setup()
-	})
-	AfterEach(func() {
-		teardown(originalHomeDir, testDir, mb)
-	})
 
 	It("bad machine name", func() {
 		name := randomString()
@@ -31,7 +21,7 @@ var _ = Describe("podman machine ssh", func() {
 	It("ssh to non-running machine", func() {
 		name := randomString()
 		i := new(initMachine)
-		session, err := mb.setName(name).setCmd(i.withImagePath(mb.imagePath)).run()
+		session, err := mb.setName(name).setCmd(i.withImage(mb.imagePath)).run()
 		Expect(err).ToNot(HaveOccurred())
 		Expect(session).To(Exit(0))
 
@@ -43,9 +33,10 @@ var _ = Describe("podman machine ssh", func() {
 	})
 
 	It("ssh to running machine and check os-type", func() {
+		wsl := testProvider.VMType() == define.WSLVirt
 		name := randomString()
 		i := new(initMachine)
-		session, err := mb.setName(name).setCmd(i.withImagePath(mb.imagePath).withNow()).run()
+		session, err := mb.setName(name).setCmd(i.withImage(mb.imagePath).withNow()).run()
 		Expect(err).ToNot(HaveOccurred())
 		Expect(session).To(Exit(0))
 
@@ -53,13 +44,22 @@ var _ = Describe("podman machine ssh", func() {
 		sshSession, err := mb.setName(name).setCmd(ssh.withSSHCommand([]string{"cat", "/etc/os-release"})).run()
 		Expect(err).ToNot(HaveOccurred())
 		Expect(sshSession).To(Exit(0))
-		Expect(sshSession.outputToString()).To(ContainSubstring("Fedora CoreOS"))
+
+		if wsl {
+			Expect(sshSession.outputToString()).To(ContainSubstring("Fedora Linux"))
+		} else {
+			Expect(sshSession.outputToString()).To(ContainSubstring("Fedora CoreOS"))
+		}
 
 		// keep exit code
 		sshSession, err = mb.setName(name).setCmd(ssh.withSSHCommand([]string{"false"})).run()
 		Expect(err).ToNot(HaveOccurred())
 		Expect(sshSession).To(Exit(1))
 		Expect(sshSession.outputToString()).To(Equal(""))
-		Expect(sshSession.errorToString()).To(Equal(""))
+
+		// WSL will often emit an error message about the ssh key and keychains
+		if !wsl {
+			Expect(sshSession.errorToString()).To(Equal(""))
+		}
 	})
 })
