@@ -16,7 +16,9 @@ import (
 	"github.com/containers/podman/v5/pkg/machine/env"
 	"github.com/containers/podman/v5/pkg/machine/ignition"
 	"github.com/containers/podman/v5/pkg/machine/lock"
+	"github.com/containers/podman/v5/pkg/machine/provider"
 	"github.com/containers/podman/v5/pkg/machine/proxyenv"
+	"github.com/containers/podman/v5/pkg/machine/shim/diskpull"
 	"github.com/containers/podman/v5/pkg/machine/vmconfigs"
 	"github.com/containers/podman/v5/utils"
 	"github.com/hashicorp/go-multierror"
@@ -126,12 +128,14 @@ func Init(opts machineDefine.InitOptions, mp vmconfigs.VMProvider) error {
 	switch mp.VMType() {
 	case machineDefine.QemuVirt:
 		imageExtension = ".qcow2"
-	case machineDefine.AppleHvVirt:
+	case machineDefine.AppleHvVirt, machineDefine.LibKrun:
 		imageExtension = ".raw"
 	case machineDefine.HyperVVirt:
 		imageExtension = ".vhdx"
+	case machineDefine.WSLVirt:
+		imageExtension = ""
 	default:
-		// do nothing
+		return fmt.Errorf("unknown VM type: %s", mp.VMType())
 	}
 
 	imagePath, err = dirs.DataDir.AppendToNewVMFile(fmt.Sprintf("%s-%s%s", opts.Name, runtime.GOARCH, imageExtension), nil)
@@ -149,7 +153,7 @@ func Init(opts machineDefine.InitOptions, mp vmconfigs.VMProvider) error {
 	// "/path
 	// "docker://quay.io/something/someManifest
 
-	if err := mp.GetDisk(opts.Image, dirs, mc); err != nil {
+	if err := diskpull.GetDisk(opts.Image, dirs, mc.ImagePath, mp.VMType(), mc.Name); err != nil {
 		return err
 	}
 
@@ -192,12 +196,15 @@ func Init(opts machineDefine.InitOptions, mp vmconfigs.VMProvider) error {
 	// copy it into the conf dir
 	if len(opts.IgnitionPath) > 0 {
 		err = ignBuilder.BuildWithIgnitionFile(opts.IgnitionPath)
-		return err
-	}
 
-	err = ignBuilder.GenerateIgnitionConfig()
-	if err != nil {
-		return err
+		if err != nil {
+			return err
+		}
+	} else {
+		err = ignBuilder.GenerateIgnitionConfig()
+		if err != nil {
+			return err
+		}
 	}
 
 	readyIgnOpts, err := mp.PrepareIgnition(mc, &ignBuilder)
@@ -228,7 +235,11 @@ func Init(opts machineDefine.InitOptions, mp vmconfigs.VMProvider) error {
 	}
 
 	cleanup := func() error {
-		return connection.RemoveConnections(mc.Name, mc.Name+"-root")
+		machines, err := provider.GetAllMachinesAndRootfulness()
+		if err != nil {
+			return err
+		}
+		return connection.RemoveConnections(machines, mc.Name, mc.Name+"-root")
 	}
 	callbackFuncs.Add(cleanup)
 
@@ -237,9 +248,10 @@ func Init(opts machineDefine.InitOptions, mp vmconfigs.VMProvider) error {
 		return err
 	}
 
-	err = ignBuilder.Build()
-	if err != nil {
-		return err
+	if len(opts.IgnitionPath) == 0 {
+		if err := ignBuilder.Build(); err != nil {
+			return err
+		}
 	}
 
 	return mc.Write()
@@ -281,7 +293,10 @@ func checkExclusiveActiveVM(provider vmconfigs.VMProvider, mc *vmconfigs.Machine
 			return err
 		}
 		if state == machineDefine.Running || state == machineDefine.Starting {
-			return fmt.Errorf("unable to start %q: machine %s: %w", mc.Name, name, machineDefine.ErrVMAlreadyRunning)
+			if mc.Name == name {
+				return fmt.Errorf("unable to start %q: machine %s: %w", mc.Name, name, machineDefine.ErrVMAlreadyRunning)
+			}
+			return fmt.Errorf("unable to start %q: machine %s is already running: %w", mc.Name, name, machineDefine.ErrMultipleActiveVM)
 		}
 	}
 	return nil
@@ -578,7 +593,12 @@ func Remove(mc *vmconfigs.MachineConfig, mp vmconfigs.VMProvider, dirs *machineD
 		}
 	}
 
-	rmFiles, genericRm, err := mc.Remove(opts.SaveIgnition, opts.SaveImage)
+	machines, err := provider.GetAllMachinesAndRootfulness()
+	if err != nil {
+		return err
+	}
+
+	rmFiles, genericRm, err := mc.Remove(machines, opts.SaveIgnition, opts.SaveImage)
 	if err != nil {
 		return err
 	}
@@ -636,38 +656,66 @@ func confirmationMessage(files []string) {
 	}
 }
 
-func Reset(dirs *machineDefine.MachineDirs, mp vmconfigs.VMProvider, mcs map[string]*vmconfigs.MachineConfig) error {
+func Reset(mps []vmconfigs.VMProvider, opts machine.ResetOptions) error {
 	var resetErrors *multierror.Error
-	for _, mc := range mcs {
-		err := Stop(mc, mp, dirs, true)
+	removeDirs := []*machineDefine.MachineDirs{}
+
+	for _, p := range mps {
+		d, err := env.GetMachineDirs(p.VMType())
 		if err != nil {
 			resetErrors = multierror.Append(resetErrors, err)
+			continue
 		}
-		_, genericRm, err := mc.Remove(false, false)
+		mcs, err := vmconfigs.LoadMachinesInDir(d)
 		if err != nil {
 			resetErrors = multierror.Append(resetErrors, err)
+			continue
 		}
-		_, providerRm, err := mp.Remove(mc)
+		removeDirs = append(removeDirs, d)
+
+		machines, err := provider.GetAllMachinesAndRootfulness()
 		if err != nil {
-			resetErrors = multierror.Append(resetErrors, err)
+			return err
 		}
 
-		if err := genericRm(); err != nil {
-			resetErrors = multierror.Append(resetErrors, err)
-		}
-		if err := providerRm(); err != nil {
-			resetErrors = multierror.Append(resetErrors, err)
+		for _, mc := range mcs {
+			err := Stop(mc, p, d, true)
+			if err != nil {
+				resetErrors = multierror.Append(resetErrors, err)
+			}
+			_, genericRm, err := mc.Remove(machines, false, false)
+			if err != nil {
+				resetErrors = multierror.Append(resetErrors, err)
+			}
+			_, providerRm, err := p.Remove(mc)
+			if err != nil {
+				resetErrors = multierror.Append(resetErrors, err)
+			}
+
+			if err := genericRm(); err != nil {
+				resetErrors = multierror.Append(resetErrors, err)
+			}
+			if err := providerRm(); err != nil {
+				resetErrors = multierror.Append(resetErrors, err)
+			}
 		}
 	}
 
 	// Delete the various directories
+	// We do this after all the provider rm's, since providers may still share the base machine dir.
 	// Note: we cannot delete the machine run dir blindly like this because
 	// other things live there like the podman.socket and so forth.
-
-	// in linux this ~/.local/share/containers/podman/machine
-	dataDirErr := utils.GuardedRemoveAll(filepath.Dir(dirs.DataDir.GetPath()))
-	// in linux this ~/.config/containers/podman/machine
-	confDirErr := utils.GuardedRemoveAll(filepath.Dir(dirs.ConfigDir.GetPath()))
-	resetErrors = multierror.Append(resetErrors, confDirErr, dataDirErr)
+	for _, dir := range removeDirs {
+		// in linux this ~/.local/share/containers/podman/machine
+		dataDirErr := utils.GuardedRemoveAll(filepath.Dir(dir.DataDir.GetPath()))
+		if !errors.Is(dataDirErr, os.ErrNotExist) {
+			resetErrors = multierror.Append(resetErrors, dataDirErr)
+		}
+		// in linux this ~/.config/containers/podman/machine
+		confDirErr := utils.GuardedRemoveAll(filepath.Dir(dir.ConfigDir.GetPath()))
+		if !errors.Is(confDirErr, os.ErrNotExist) {
+			resetErrors = multierror.Append(resetErrors, confDirErr)
+		}
+	}
 	return resetErrors.ErrorOrNil()
 }
