@@ -8,7 +8,7 @@ import (
 
 	nettypes "github.com/containers/common/libnetwork/types"
 	"github.com/containers/image/v5/manifest"
-	"github.com/containers/podman/v4/libpod/define"
+	"github.com/containers/podman/v5/libpod/define"
 	"github.com/containers/storage/types"
 	spec "github.com/opencontainers/runtime-spec/specs-go"
 )
@@ -52,20 +52,21 @@ type ContainerBasicConfig struct {
 	Command []string `json:"command,omitempty"`
 	// EnvHost indicates that the host environment should be added to container
 	// Optional.
-	EnvHost bool `json:"env_host,omitempty"`
+	EnvHost *bool `json:"env_host,omitempty"`
 	// EnvHTTPProxy indicates that the http host proxy environment variables
 	// should be added to container
 	// Optional.
-	HTTPProxy bool `json:"httpproxy,omitempty"`
+	HTTPProxy *bool `json:"httpproxy,omitempty"`
 	// Env is a set of environment variables that will be set in the
 	// container.
 	// Optional.
 	Env map[string]string `json:"env,omitempty"`
 	// Terminal is whether the container will create a PTY.
 	// Optional.
-	Terminal bool `json:"terminal,omitempty"`
+	Terminal *bool `json:"terminal,omitempty"`
 	// Stdin is whether the container will keep its STDIN open.
-	Stdin bool `json:"stdin,omitempty"`
+	// Optional.
+	Stdin *bool `json:"stdin,omitempty"`
 	// Labels are key-value pairs that are used to add metadata to
 	// containers.
 	// Optional.
@@ -90,6 +91,7 @@ type ContainerBasicConfig struct {
 	// Timeout is a maximum time in seconds the container will run before
 	// main process is sent SIGKILL.
 	// If 0 is used, signal will not be sent. Container can run indefinitely
+	// if they do not stop after the default termination signal.
 	// Optional.
 	Timeout uint `json:"timeout,omitempty"`
 	// LogConfiguration describes the logging for a container including
@@ -101,15 +103,6 @@ type ContainerBasicConfig struct {
 	// If not given, a default location will be used.
 	// Optional.
 	ConmonPidFile string `json:"conmon_pid_file,omitempty"`
-	// RawImageName is the user-specified and unprocessed input referring
-	// to a local or a remote image.
-	RawImageName string `json:"raw_image_name,omitempty"`
-	// ImageOS is the user-specified image OS
-	ImageOS string `json:"image_os,omitempty"`
-	// ImageArch is the user-specified image architecture
-	ImageArch string `json:"image_arch,omitempty"`
-	// ImageVariant is the user-specified image variant
-	ImageVariant string `json:"image_variant,omitempty"`
 	// RestartPolicy is the container's restart policy - an action which
 	// will be taken when the container exits.
 	// If not given, the default policy, which does nothing, will be used.
@@ -140,10 +133,8 @@ type ContainerBasicConfig struct {
 	// "container" - let the OCI runtime deal with it, advertise conmon's MAINPID
 	// "conmon-only" - advertise conmon's MAINPID, send READY when started, don't pass to OCI
 	// "ignore" - unset NOTIFY_SOCKET
-	SdNotifyMode string `json:"sdnotifyMode,omitempty"`
-	// Namespace is the libpod namespace the container will be placed in.
 	// Optional.
-	Namespace string `json:"namespace,omitempty"`
+	SdNotifyMode string `json:"sdnotifyMode,omitempty"`
 	// PidNS is the container's PID namespace.
 	// It defaults to private.
 	// Mandatory.
@@ -165,8 +156,15 @@ type ContainerBasicConfig struct {
 	// Sysctl sets kernel parameters for the container
 	Sysctl map[string]string `json:"sysctl,omitempty"`
 	// Remove indicates if the container should be removed once it has been started
-	// and exits
-	Remove bool `json:"remove,omitempty"`
+	// and exits.
+	// Optional.
+	Remove *bool `json:"remove,omitempty"`
+	// RemoveImage indicates that the container should remove the image it
+	// was created from after it exits.
+	// Only allowed if Remove is set to true and Image, not Rootfs, is in
+	// use.
+	// Optional.
+	RemoveImage *bool `json:"removeImage,omitempty"`
 	// ContainerCreateCommand is the command that was used to create this
 	// container.
 	// This will be shown in the output of Inspect() on the container, and
@@ -180,6 +178,11 @@ type ContainerBasicConfig struct {
 	// set tags as `json:"-"` for not supported remote
 	// Optional.
 	PreserveFDs uint `json:"-"`
+	// PreserveFD is a list of additional file descriptors (in addition
+	// to 0, 1, 2) that will be passed to the executed process.
+	// set tags as `json:"-"` for not supported remote
+	// Optional.
+	PreserveFD []uint `json:"-"`
 	// Timezone is the timezone inside the container.
 	// Local means it has the same timezone as the host machine
 	// Optional.
@@ -189,36 +192,41 @@ type ContainerBasicConfig struct {
 	// container. Dependencies can be specified by name or full/partial ID.
 	// Optional.
 	DependencyContainers []string `json:"dependencyContainers,omitempty"`
-	// PidFile is the file that saves container process id.
-	// set tags as `json:"-"` for not supported remote
+	// PidFile is the file that saves container's PID.
+	// Not supported for remote clients, so not serialized in specgen JSON.
 	// Optional.
 	PidFile string `json:"-"`
 	// EnvSecrets are secrets that will be set as environment variables
 	// Optional.
 	EnvSecrets map[string]string `json:"secret_env,omitempty"`
 	// InitContainerType describes if this container is an init container
-	// and if so, what type: always or once
+	// and if so, what type: always or once.
+	// Optional.
 	InitContainerType string `json:"init_container_type"`
 	// Personality allows users to configure different execution domains.
 	// Execution domains tell Linux how to map signal numbers into signal actions.
 	// The execution domain system allows Linux to provide limited support
 	// for binaries compiled under other UNIX-like operating systems.
+	// Optional.
 	Personality *spec.LinuxPersonality `json:"personality,omitempty"`
 	// EnvMerge takes the specified environment variables from image and preprocess them before injecting them into the
 	// container.
+	// Optional.
 	EnvMerge []string `json:"envmerge,omitempty"`
-	// UnsetEnv unsets the specified default environment variables from the image or from buildin or containers.conf
+	// UnsetEnv unsets the specified default environment variables from the image or from built-in or containers.conf
 	// Optional.
 	UnsetEnv []string `json:"unsetenv,omitempty"`
-	// UnsetEnvAll unsetall default environment variables from the image or from buildin or containers.conf
-	// UnsetEnvAll unsets all default environment variables from the image or from buildin
+	// UnsetEnvAll unsetall default environment variables from the image or from built-in or containers.conf
+	// UnsetEnvAll unsets all default environment variables from the image or from built-in
 	// Optional.
-	UnsetEnvAll bool `json:"unsetenvall,omitempty"`
+	UnsetEnvAll *bool `json:"unsetenvall,omitempty"`
 	// Passwd is a container run option that determines if we are validating users/groups before running the container
 	Passwd *bool `json:"manage_password,omitempty"`
-	// PasswdEntry specifies arbitrary data to append to a file.
+	// PasswdEntry specifies an arbitrary string to append to the container's /etc/passwd file.
+	// Optional.
 	PasswdEntry string `json:"passwd_entry,omitempty"`
-	// GroupEntry specifies arbitrary data to append to a file.
+	// GroupEntry specifies an arbitrary string to append to the container's /etc/group file.
+	// Optional.
 	GroupEntry string `json:"group_entry,omitempty"`
 }
 
@@ -231,15 +239,33 @@ type ContainerStorageConfig struct {
 	// Conflicts with Rootfs.
 	// At least one of Image or Rootfs must be specified.
 	Image string `json:"image"`
+	// RawImageName is the user-specified and unprocessed input referring
+	// to a local or a remote image.
+	// Optional, but strongly encouraged to be set if Image is set.
+	RawImageName string `json:"raw_image_name,omitempty"`
+	// ImageOS is the user-specified OS of the image.
+	// Used to select a different variant from a manifest list.
+	// Optional.
+	ImageOS string `json:"image_os,omitempty"`
+	// ImageArch is the user-specified image architecture.
+	// Used to select a different variant from a manifest list.
+	// Optional.
+	ImageArch string `json:"image_arch,omitempty"`
+	// ImageVariant is the user-specified image variant.
+	// Used to select a different variant from a manifest list.
+	// Optional.
+	ImageVariant string `json:"image_variant,omitempty"`
 	// Rootfs is the path to a directory that will be used as the
 	// container's root filesystem. No modification will be made to the
 	// directory, it will be directly mounted into the container as root.
 	// Conflicts with Image.
 	// At least one of Image or Rootfs must be specified.
 	Rootfs string `json:"rootfs,omitempty"`
-	// RootfsOverlay tells if rootfs is actually an overlay on top of base path
-	RootfsOverlay bool `json:"rootfs_overlay,omitempty"`
-	// RootfsMapping specifies if there are mappings to apply to the rootfs.
+	// RootfsOverlay tells if rootfs is actually an overlay on top of base path.
+	// Optional.
+	RootfsOverlay *bool `json:"rootfs_overlay,omitempty"`
+	// RootfsMapping specifies if there are UID/GID mappings to apply to the rootfs.
+	// Optional.
 	RootfsMapping *string `json:"rootfs_mapping,omitempty"`
 	// ImageVolumeMode indicates how image volumes will be created.
 	// Supported modes are "ignore" (do not create), "tmpfs" (create as
@@ -252,10 +278,12 @@ type ContainerStorageConfig struct {
 	// may optionally be followed by a : and then one or more
 	// comma-separated options. Valid options are 'ro', 'rw', and 'z'.
 	// Options will be used for all volumes sourced from the container.
+	// Optional.
 	VolumesFrom []string `json:"volumes_from,omitempty"`
 	// Init specifies that an init binary will be mounted into the
 	// container, and will be used as PID1.
-	Init bool `json:"init,omitempty"`
+	// Optional.
+	Init *bool `json:"init,omitempty"`
 	// InitPath specifies the path to the init binary that will be added if
 	// Init is specified above. If not specified, the default set in the
 	// Libpod config will be used. Ignored if Init above is not set.
@@ -283,7 +311,8 @@ type ContainerStorageConfig struct {
 	// DeviceCgroupRule are device cgroup rules that allow containers
 	// to use additional types of devices.
 	DeviceCgroupRule []spec.LinuxDeviceCgroup `json:"device_cgroup_rule,omitempty"`
-	// DevicesFrom is a way to ensure your container inherits device specific information from another container
+	// DevicesFrom specifies that this container will mount the device(s) from other container(s).
+	// Optional.
 	DevicesFrom []string `json:"devices_from,omitempty"`
 	// HostDeviceList is used to recreate the mounted device on inherited containers
 	HostDeviceList []spec.LinuxDevice `json:"host_device_list,omitempty"`
@@ -307,7 +336,7 @@ type ContainerStorageConfig struct {
 	// Create the working directory if it doesn't exist.
 	// If unset, it doesn't create it.
 	// Optional.
-	CreateWorkingDir bool `json:"create_working_dir,omitempty"`
+	CreateWorkingDir *bool `json:"create_working_dir,omitempty"`
 	// StorageOpts is the container's storage options
 	// Optional.
 	StorageOpts map[string]string `json:"storage_opts,omitempty"`
@@ -320,10 +349,12 @@ type ContainerStorageConfig struct {
 	Secrets []Secret `json:"secrets,omitempty"`
 	// Volatile specifies whether the container storage can be optimized
 	// at the cost of not syncing all the dirty files in memory.
-	Volatile bool `json:"volatile,omitempty"`
+	// Optional.
+	Volatile *bool `json:"volatile,omitempty"`
 	// ChrootDirs is an additional set of directories that need to be
 	// treated as root directories. Standard bind mounts will be mounted
 	// into paths relative to these directories.
+	// Optional.
 	ChrootDirs []string `json:"chroot_directories,omitempty"`
 }
 
@@ -338,7 +369,8 @@ type ContainerSecurityConfig struct {
 	//   (Though SELinux can be manually re-enabled).
 	// TODO: this conflicts with things.
 	// TODO: this does more.
-	Privileged bool `json:"privileged,omitempty"`
+	// Optional.
+	Privileged *bool `json:"privileged,omitempty"`
 	// User is the user the container will be run as.
 	// Can be given as a UID or a username; if a username, it will be
 	// resolved within the container, using the container's /etc/passwd.
@@ -377,7 +409,8 @@ type ContainerSecurityConfig struct {
 	// NoNewPrivileges is whether the container will set the no new
 	// privileges flag on create, which disables gaining additional
 	// privileges (e.g. via setuid) in the container.
-	NoNewPrivileges bool `json:"no_new_privileges,omitempty"`
+	// Optional.
+	NoNewPrivileges *bool `json:"no_new_privileges,omitempty"`
 	// UserNS is the container's user namespace.
 	// It defaults to host, indicating that no user namespace will be
 	// created.
@@ -389,15 +422,18 @@ type ContainerSecurityConfig struct {
 	// Required if UserNS is private.
 	IDMappings *types.IDMappingOptions `json:"idmappings,omitempty"`
 	// ReadOnlyFilesystem indicates that everything will be mounted
-	// as read-only
-	ReadOnlyFilesystem bool `json:"read_only_filesystem,omitempty"`
+	// as read-only.
+	// Optional.
+	ReadOnlyFilesystem *bool `json:"read_only_filesystem,omitempty"`
 	// ReadWriteTmpfs indicates that when running with a ReadOnlyFilesystem
-	// mount temporary file systems
-	ReadWriteTmpfs bool `json:"read_write_tmpfs,omitempty"`
+	// mount temporary file systems.
+	// Optional.
+	ReadWriteTmpfs *bool `json:"read_write_tmpfs,omitempty"`
 
 	// LabelNested indicates whether or not the container is allowed to
-	// run fully nested containers including labelling
-	LabelNested bool `json:"label_nested,omitempty"`
+	// run fully nested containers including SELinux labelling.
+	// Optional.
+	LabelNested *bool `json:"label_nested,omitempty"`
 
 	// Umask is the umask the init process of the container will be run with.
 	Umask string `json:"umask,omitempty"`
@@ -407,8 +443,10 @@ type ContainerSecurityConfig struct {
 	// given in addition to the default list.
 	// Optional
 	Mask []string `json:"mask,omitempty"`
-	// Unmask is the path we want to unmask in the container. To override
-	// all the default paths that are masked, set unmask=ALL.
+	// Unmask a path in the container. Some paths are masked by default,
+	// preventing them from being accessed within the container; this undoes
+	// that masking. If ALL is passed, all paths will be unmasked.
+	// Optional.
 	Unmask []string `json:"unmask,omitempty"`
 }
 
@@ -419,8 +457,9 @@ type ContainerCgroupConfig struct {
 	// It defaults to private.
 	// Mandatory.
 	CgroupNS Namespace `json:"cgroupns,omitempty"`
-	// CgroupsMode sets a policy for how cgroups will be created in the
+	// CgroupsMode sets a policy for how cgroups will be created for the
 	// container, including the ability to disable creation entirely.
+	// Optional.
 	CgroupsMode string `json:"cgroups_mode,omitempty"`
 	// CgroupParent is the container's Cgroup parent.
 	// If not set, the default for the current cgroup driver will be used.
@@ -444,7 +483,8 @@ type ContainerNetworkConfig struct {
 	// This is based on ports set in Expose below, and any ports specified
 	// by the Image (if one is given).
 	// Only available if NetNS is set to Bridge or Slirp.
-	PublishExposedPorts bool `json:"publish_image_ports,omitempty"`
+	// Optional.
+	PublishExposedPorts *bool `json:"publish_image_ports,omitempty"`
 	// Expose is a number of ports that will be forwarded to the container
 	// if PublishExposedPorts is set.
 	// Expose is a map of uint16 (port number) to a string representing
@@ -461,6 +501,7 @@ type ContainerNetworkConfig struct {
 	// network interface name for this container on the specific network.
 	// If the map is empty and the bridge network mode is set the container
 	// will be joined to the default network.
+	// Optional.
 	Networks map[string]nettypes.PerNetworkOptions
 	// CNINetworks is a list of CNI networks to join the container to.
 	// If this list is empty, the default CNI network will be joined
@@ -473,7 +514,8 @@ type ContainerNetworkConfig struct {
 	// UseImageResolvConf indicates that resolv.conf should not be managed
 	// by Podman, but instead sourced from the image.
 	// Conflicts with DNSServer, DNSSearch, DNSOption.
-	UseImageResolvConf bool `json:"use_image_resolve_conf,omitempty"`
+	// Optional.
+	UseImageResolvConf *bool `json:"use_image_resolve_conf,omitempty"`
 	// DNSServers is a set of DNS servers that will be used in the
 	// container's resolv.conf, replacing the host's DNS Servers which are
 	// used by default.
@@ -495,13 +537,15 @@ type ContainerNetworkConfig struct {
 	// UseImageHosts indicates that /etc/hosts should not be managed by
 	// Podman, and instead sourced from the image.
 	// Conflicts with HostAdd.
-	// Do not set omitempty here, if this is false it should be set to not get
-	// the server default.
-	// Ideally this would be a pointer so we could differentiate between an
-	// explicitly false/true and unset (containers.conf default). However
-	// specgen is stable so we can not change this right now.
-	// TODO (5.0): change to pointer
-	UseImageHosts bool `json:"use_image_hosts"`
+	// Optional.
+	UseImageHosts *bool `json:"use_image_hosts,omitempty"`
+	// BaseHostsFile is the path to a hosts file, the entries from this file
+	// are added to the containers hosts file. As special value "image" is
+	// allowed which uses the /etc/hosts file from within the image and "none"
+	// which uses no base file at all. If it is empty we should default
+	// to the base_hosts_file configuration in containers.conf.
+	// Optional.
+	BaseHostsFile string `json:"base_hosts_file,omitempty"`
 	// HostAdd is a set of hosts which will be added to the container's
 	// /etc/hosts file.
 	// Conflicts with UseImageHosts.
@@ -514,6 +558,10 @@ type ContainerNetworkConfig struct {
 
 // ContainerResourceConfig contains information on container resource limits.
 type ContainerResourceConfig struct {
+	// IntelRdt defines the Intel RDT CAT Class of Service (COS) that all processes
+	// of the container should run in.
+	// Optional.
+	IntelRdt *spec.LinuxIntelRdt `json:"intelRdt,omitempty"`
 	// ResourceLimits are resource limits to apply to the container.,
 	// Can only be set as root on cgroups v1 systems, but can be set as
 	// rootless as well for cgroups v2.
@@ -540,10 +588,6 @@ type ContainerResourceConfig struct {
 	// that are used to configure cgroup v2.
 	// Optional.
 	CgroupConf map[string]string `json:"unified,omitempty"`
-	// CPU period of the cpuset, determined by --cpus
-	CPUPeriod uint64 `json:"cpu_period,omitempty"`
-	// CPU quota of the cpuset, determined by --cpus
-	CPUQuota int64 `json:"cpu_quota,omitempty"`
 }
 
 // ContainerHealthCheckConfig describes a container healthcheck with attributes
@@ -555,6 +599,14 @@ type ContainerHealthCheckConfig struct {
 	// Requires that HealthConfig be set.
 	// Optional.
 	StartupHealthConfig *define.StartupHealthCheck `json:"startupHealthConfig,omitempty"`
+	// HealthLogDestination defines the destination where the log is stored
+	HealthLogDestination string `json:"healthLogDestination,omitempty"`
+	// HealthMaxLogCount is maximum number of attempts in the HealthCheck log file.
+	// ('0' value means an infinite number of attempts in the log file)
+	HealthMaxLogCount uint `json:"healthMaxLogCount,omitempty"`
+	// HealthMaxLogSize is the maximum length in characters of stored HealthCheck log
+	// ("0" value means an infinite log length)
+	HealthMaxLogSize uint `json:"healthMaxLogSize,omitempty"`
 }
 
 // SpecGenerator creates an OCI spec and Libpod configuration options to create
@@ -574,6 +626,13 @@ type SpecGenerator struct {
 	cacheLibImage
 }
 
+func (s *SpecGenerator) IsPrivileged() bool {
+	if s.Privileged != nil {
+		return *s.Privileged
+	}
+	return false
+}
+
 func (s *SpecGenerator) IsInitContainer() bool {
 	return len(s.InitContainerType) != 0
 }
@@ -607,7 +666,8 @@ func NewSpecGenerator(arg string, rootfs bool) *SpecGenerator {
 		if lastColonIndex != -1 {
 			lastPart := csc.Rootfs[lastColonIndex+1:]
 			if lastPart == "O" {
-				csc.RootfsOverlay = true
+				localTrue := true
+				csc.RootfsOverlay = &localTrue
 				csc.Rootfs = csc.Rootfs[:lastColonIndex]
 			} else if lastPart == "idmap" || strings.HasPrefix(lastPart, "idmap=") {
 				csc.RootfsMapping = &lastPart
@@ -619,13 +679,25 @@ func NewSpecGenerator(arg string, rootfs bool) *SpecGenerator {
 	}
 	return &SpecGenerator{
 		ContainerStorageConfig: csc,
+		ContainerHealthCheckConfig: ContainerHealthCheckConfig{
+			HealthLogDestination: define.DefaultHealthCheckLocalDestination,
+			HealthMaxLogCount:    define.DefaultHealthMaxLogCount,
+			HealthMaxLogSize:     define.DefaultHealthMaxLogSize,
+		},
 	}
 }
 
 // NewSpecGenerator returns a SpecGenerator struct given one of two mandatory inputs
 func NewSpecGeneratorWithRootfs(rootfs string) *SpecGenerator {
 	csc := ContainerStorageConfig{Rootfs: rootfs}
-	return &SpecGenerator{ContainerStorageConfig: csc}
+	return &SpecGenerator{
+		ContainerStorageConfig: csc,
+		ContainerHealthCheckConfig: ContainerHealthCheckConfig{
+			HealthLogDestination: define.DefaultHealthCheckLocalDestination,
+			HealthMaxLogCount:    define.DefaultHealthMaxLogCount,
+			HealthMaxLogSize:     define.DefaultHealthMaxLogSize,
+		},
+	}
 }
 
 func StringSlicesEqual(a, b []string) bool {
