@@ -1,3 +1,5 @@
+//go:build linux || freebsd
+
 package integration
 
 import (
@@ -428,8 +430,7 @@ var _ = Describe("Podman network", func() {
 		// check if pod is deleted
 		session = podmanTest.Podman([]string{"pod", "exists", podID})
 		session.WaitWithDefaultTimeout()
-		Expect(session).Should(Exit(1))
-		Expect(session.ErrorToString()).To(Equal(""))
+		Expect(session).Should(ExitWithError(1, ""))
 
 		// check if net is deleted
 		session = podmanTest.Podman([]string{"network", "ls"})
@@ -548,7 +549,6 @@ var _ = Describe("Podman network", func() {
 	})
 
 	for _, opt := range []string{"-o=parent=lo", "--interface-name=lo"} {
-		opt := opt
 		It(fmt.Sprintf("podman network create/remove macvlan as driver (-d) with %s", opt), func() {
 			net := "macvlan" + stringid.GenerateRandomID()
 			nc := podmanTest.Podman([]string{"network", "create", "-d", "macvlan", opt, net})
@@ -617,8 +617,7 @@ var _ = Describe("Podman network", func() {
 
 		session = podmanTest.Podman([]string{"network", "exists", stringid.GenerateRandomID()})
 		session.WaitWithDefaultTimeout()
-		Expect(session).Should(Exit(1))
-		Expect(session.ErrorToString()).To(Equal(""))
+		Expect(session).Should(ExitWithError(1, ""))
 	})
 
 	It("podman network create macvlan with network info and options", func() {
@@ -696,7 +695,7 @@ var _ = Describe("Podman network", func() {
 		Expect(listAgain.OutputToStringArray()).Should(ContainElement("podman"))
 	})
 
-	It("podman network prune", func() {
+	It("podman network prune", Serial, func() {
 		useCustomNetworkDir(podmanTest, tempdir)
 		// Create two networks
 		// Check they are there
